@@ -32,6 +32,7 @@ pub trait AudioNode {
     /// Reset the input state of the component to an initial state where it has
     /// not processed any samples. In other words, reset time to zero.
     /// The sample rate can be set optionally. The default sample rate is 44.1 kHz.
+    /// The default implementation does nothing.
     fn reset(&mut self, _sample_rate: Option<f64>) {}
 
     /// Process one sample.
@@ -65,6 +66,7 @@ pub trait AudioNode {
 
     /// Set node hash. Override this to use the hash.
     /// This is called from `ping`. It should not be called by users.
+    /// The default implementation does nothing.
     fn set_hash(&mut self, _hash: u64) {}
 
     /// Ping contained `AudioNode`s to obtain a deterministic pseudorandom hash.
@@ -491,12 +493,17 @@ where
     }
 }
 
+/// Provides binary operator implementations to the `Binop` node.
 pub trait FrameBinop<T: Float, N: Size<T>> {
+    /// Do binary op (x op y) channelwise.
     fn binop(x: &Frame<T, N>, y: &Frame<T, N>) -> Frame<T, N>;
+    /// Do binary op (x op y) on signals.
     fn propagate(x: Signal, y: Signal) -> Signal;
-    /// Do binary op (x op y) in-place.
+    /// Do binary op (x op y) in-place lengthwise.
     fn assign(size: usize, x: &mut [T], y: &[T]);
 }
+
+/// Addition operator.
 #[derive(Default)]
 pub struct FrameAdd<T: Float, N: Size<T>> {
     _marker: PhantomData<(T, N)>,
@@ -524,6 +531,7 @@ impl<T: Float, N: Size<T>> FrameBinop<T, N> for FrameAdd<T, N> {
     }
 }
 
+/// Subtraction operator.
 #[derive(Default)]
 pub struct FrameSub<T: Float, N: Size<T>> {
     _marker: PhantomData<(T, N)>,
@@ -551,6 +559,7 @@ impl<T: Float, N: Size<T>> FrameBinop<T, N> for FrameSub<T, N> {
     }
 }
 
+/// Multiplication operator.
 #[derive(Default)]
 pub struct FrameMul<T: Float, N: Size<T>> {
     _marker: PhantomData<(T, N)>,
@@ -597,7 +606,7 @@ impl<T: Float, N: Size<T>> FrameBinop<T, N> for FrameMul<T, N> {
 
 /// Combine outputs of two nodes with a binary operation.
 /// Inputs are disjoint.
-/// Outputs are combined channel-wise.
+/// Outputs are combined channelwise.
 /// The nodes must have the same number of outputs.
 pub struct Binop<T, B, X, Y>
 where
@@ -721,13 +730,17 @@ where
     }
 }
 
+/// Provides unary operator implementations to the `Unop` node.
 pub trait FrameUnop<T: Float, N: Size<T>> {
+    /// Do unary op channelwise.
     fn unop(x: &Frame<T, N>) -> Frame<T, N>;
+    /// Do unary op on signal.
     fn propagate(x: Signal) -> Signal;
-    /// Do unary op in-place.
+    /// Do unary op in-place lengthwise.
     fn assign(size: usize, x: &mut [T]);
 }
 
+/// Negation operator.
 #[derive(Default)]
 pub struct FrameNeg<T: Float, N: Size<T>> {
     _marker: PhantomData<(T, N)>,
@@ -859,13 +872,13 @@ where
     }
 }
 
+/// Map any number of channels.
 pub struct Map<T, M, I, O> {
     f: M,
     routing: Routing,
     _marker: PhantomData<(T, I, O)>,
 }
 
-/// Map any number of channels.
 impl<T, M, I, O> Map<T, M, I, O>
 where
     T: Float,
@@ -2071,3 +2084,45 @@ where
         output
     }
 }
+
+/// Swap stereo channels.
+#[derive(Default)]
+pub struct Swap<T> {
+    _marker: PhantomData<T>,
+}
+
+impl<T: Float> Swap<T> {
+    pub fn new() -> Self {
+        Swap::default()
+    }
+}
+
+impl<T: Float> AudioNode for Swap<T> {
+    const ID: u64 = 45;
+    type Sample = T;
+    type Inputs = U2;
+    type Outputs = U2;
+
+    #[inline]
+    fn tick(
+        &mut self,
+        input: &Frame<Self::Sample, Self::Inputs>,
+    ) -> Frame<Self::Sample, Self::Outputs> {
+        [input[1], input[0]].into()
+    }
+    fn process(
+        &mut self,
+        size: usize,
+        input: &[&[Self::Sample]],
+        output: &mut [&mut [Self::Sample]],
+    ) {
+        output[0][..size].clone_from_slice(&input[1][..size]);
+        output[1][..size].clone_from_slice(&input[0][..size]);
+    }
+    fn route(&self, input: &SignalFrame, _frequency: f64) -> SignalFrame {
+        let mut output = new_signal_frame(self.outputs());
+        output[0] = input[1];
+        output[1] = input[0];
+        output
+    }
+}
