@@ -173,18 +173,28 @@ pub fn multipass<U: Size<f64>>() -> An<Pass<f64, U>> {
     An(Pass::new())
 }
 
-/// Mono sink.
+/// Mono sink. Input is discarded.
 #[inline]
 pub fn sink() -> An<Sink<f64, U1>> {
     An(Sink::new())
 }
 
-/// Multichannel sink.
+/// Multichannel sink. Inputs are discarded.
 #[inline]
 pub fn multisink<U: Size<f64>>() -> An<Sink<f64, U>> {
     An(Sink::new())
 }
 
+/// Swap stereo channels.
+/// - Input 0: left channel.
+/// - Input 1: right channel.
+/// - Output 0: right channel input.
+/// - Output 1: left channel input.
+#[inline]
+pub fn swap() -> An<Swap<f64>> {
+    An(Swap::new())
+}
+
 /// Sine oscillator.
 /// - Input 0: frequency (Hz)
 /// - Output 0: sine wave
@@ -274,6 +284,33 @@ pub fn lowpole_hz(
     super::prelude::lowpole_hz(f)
 }
 
+/// Allpole filter with a configurable delay (delay > 0) in samples at DC.
+/// - Input 0: audio
+/// - Output 0: filtered audio
+#[inline]
+pub fn allpole_delay(delay_in_samples: f64) -> An<Allpole<f64, f64>> {
+    An(Allpole::new(DEFAULT_SR, delay_in_samples))
+}
+
+/// One-pole, one-zero highpass filter (1st order).
+/// - Input 0: audio
+/// - Input 1: cutoff frequency (Hz)
+/// - Output 0: filtered audio
+#[inline]
+pub fn highpole() -> An<Highpole<f64, f64>> {
+    An(Highpole::new(DEFAULT_SR, 440.0))
+}
+
+/// One-pole, one-zero highpass filter (1st order) with fixed cutoff frequency f.
+/// - Input 0: audio
+/// - Output 0: filtered audio
+#[inline]
+pub fn highpole_hz(
+    f: f64,
+) -> An<Pipe<f64, Stack<f64, Pass<f64, U1>, Constant<f64, U1>>, Highpole<f64, f64>>> {
+    (pass() | constant(f)) >> An(Highpole::new(DEFAULT_SR, f))
+}
+
 /// Constant-gain bandpass resonator.
 /// - Input 0: audio
 /// - Input 1: cutoff frequency (Hz)
@@ -306,7 +343,7 @@ where
     R: ConstantFrame<Sample = f64>,
     R::Size: Size<f64>,
 {
-    super::prelude::envelope(f)
+    An(Envelope::new(0.002, DEFAULT_SR, f))
 }
 
 /// Control envelope from time-varying function `f(t)` with `t` in seconds.
@@ -320,7 +357,7 @@ where
     R: ConstantFrame<Sample = f64>,
     R::Size: Size<f64>,
 {
-    super::prelude::envelope(f)
+    An(Envelope::new(0.002, DEFAULT_SR, f))
 }
 
 /// Maximum Length Sequence noise generator from an `n`-bit sequence.
@@ -1090,3 +1127,12 @@ pub fn highshelf_q(
 {
     super::prelude::highshelf_q::<f64, f64>(q, gain)
 }
+
+/// Pulse wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: pulse duty cycle in 0...1
+/// - Output 0: pulse wave
+#[inline]
+pub fn pulse() -> An<super::prelude::PulseWave<f64>> {
+    super::prelude::pulse()
+}
