@@ -173,18 +173,28 @@ pub fn multipass<T: Float, U: Size<T>>() -> An<Pass<T, U>> {
     An(Pass::new())
 }
 
-/// Mono sink.
+/// Mono sink. Input is discarded.
 #[inline]
 pub fn sink<T: Float>() -> An<Sink<T, U1>> {
     An(Sink::new())
 }
 
-/// Multichannel sink.
+/// Multichannel sink. Inputs are discarded.
 #[inline]
 pub fn multisink<T: Float, U: Size<T>>() -> An<Sink<T, U>> {
     An(Sink::new())
 }
 
+/// Swap stereo channels.
+/// - Input 0: left channel.
+/// - Input 1: right channel.
+/// - Output 0: right channel input.
+/// - Output 1: left channel input.
+#[inline]
+pub fn swap<T: Float>() -> An<Swap<T>> {
+    An(Swap::new())
+}
+
 /// Sine oscillator.
 /// - Input 0: frequency (Hz)
 /// - Output 0: sine wave
@@ -292,7 +302,34 @@ pub fn lowpole<T: Float, F: Real>() -> An<Lowpole<T, F>> {
 pub fn lowpole_hz<T: Float, F: Real>(
     f: T,
 ) -> An<Pipe<T, Stack<T, Pass<T, U1>, Constant<T, U1>>, Lowpole<T, F>>> {
-    (pass::<T>() | constant(f)) >> An(Lowpole::<T, F>::new(convert(DEFAULT_SR), convert(f)))
+    (pass::<T>() | constant(f)) >> An(Lowpole::<T, F>::new(DEFAULT_SR, convert(f)))
+}
+
+/// Allpole filter with a configurable delay (delay > 0) in samples at DC.
+/// - Input 0: audio
+/// - Output 0: filtered audio
+#[inline]
+pub fn allpole_delay<T: Float, F: Float>(delay_in_samples: F) -> An<Allpole<T, F>> {
+    An(Allpole::new(DEFAULT_SR, delay_in_samples))
+}
+
+/// One-pole, one-zero highpass filter (1st order).
+/// - Input 0: audio
+/// - Input 1: cutoff frequency (Hz)
+/// - Output 0: filtered audio
+#[inline]
+pub fn highpole<T: Float, F: Real>() -> An<Highpole<T, F>> {
+    An(Highpole::new(DEFAULT_SR, F::new(440)))
+}
+
+/// One-pole, one-zero highpass filter (1st order) with fixed cutoff frequency f.
+/// - Input 0: audio
+/// - Output 0: filtered audio
+#[inline]
+pub fn highpole_hz<T: Float, F: Real>(
+    f: T,
+) -> An<Pipe<T, Stack<T, Pass<T, U1>, Constant<T, U1>>, Highpole<T, F>>> {
+    (pass::<T>() | constant(f)) >> An(Highpole::new(DEFAULT_SR, convert(f)))
 }
 
 /// Constant-gain bandpass resonator.
@@ -360,7 +397,7 @@ where
     R::Size: Size<F>,
     R::Size: Size<T>,
 {
-    envelope(f)
+    An(Envelope::new(F::from_f64(0.002), DEFAULT_SR, f))
 }
 
 /// Maximum Length Sequence noise generator from an `n`-bit sequence.
@@ -920,18 +957,24 @@ where
 }
 
 /// Saw wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: saw wave
 #[inline]
 pub fn saw<T: Float>() -> An<WaveSynth<'static, T, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &SAW_TABLE))
 }
 
 /// Square wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: square wave
 #[inline]
 pub fn square<T: Float>() -> An<WaveSynth<'static, T, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &SQUARE_TABLE))
 }
 
 /// Triangle wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Output 0: triangle wave
 #[inline]
 pub fn triangle<T: Float>() -> An<WaveSynth<'static, T, U1>> {
     An(WaveSynth::new(DEFAULT_SR, &TRIANGLE_TABLE))
@@ -1453,3 +1496,115 @@ pub fn highshelf_q<T: Float, F: Real>(
             },
         ))
 }
+
+/// Pulse wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: pulse duty cycle in 0...1
+/// - Output 0: pulse wave
+pub struct PulseWave<T: Float> {
+    pulse: An<
+        Pipe<
+            T,
+            Pipe<
+                T,
+                Stack<T, WaveSynth<'static, T, U2>, Pass<T, U1>>,
+                Stack<
+                    T,
+                    Pass<T, U1>,
+                    Pipe<
+                        T,
+                        Binop<T, FrameAdd<T, U1>, Pass<T, U1>, Pass<T, U1>>,
+                        PhaseSynth<'static, T>,
+                    >,
+                >,
+            >,
+            Binop<T, FrameSub<T, U1>, Pass<T, U1>, Pass<T, U1>>,
+        >,
+    >,
+}
+
+#[allow(clippy::new_without_default)]
+impl<T: Float> PulseWave<T> {
+    pub fn new() -> Self {
+        Self {
+            pulse: (An(WaveSynth::<'static, T, U2>::new(DEFAULT_SR, &SAW_TABLE)) | pass())
+                >> (pass()
+                    | (pass() + pass())
+                        >> An(PhaseSynth::<'static, T>::new(DEFAULT_SR, &SAW_TABLE)))
+                >> pass() - pass(),
+        }
+    }
+}
+
+impl<T: Float> AudioNode for PulseWave<T> {
+    const ID: u64 = 44;
+    type Sample = T;
+    type Inputs = U2;
+    type Outputs = U1;
+
+    fn reset(&mut self, sample_rate: Option<f64>) {
+        self.pulse.reset(sample_rate);
+    }
+    #[inline]
+    fn tick(
+        &mut self,
+        input: &Frame<Self::Sample, Self::Inputs>,
+    ) -> Frame<Self::Sample, Self::Outputs> {
+        self.pulse.tick(input)
+    }
+    fn process(
+        &mut self,
+        size: usize,
+        input: &[&[Self::Sample]],
+        output: &mut [&mut [Self::Sample]],
+    ) {
+        self.pulse.process(size, input, output);
+    }
+    fn route(&self, input: &SignalFrame, frequency: f64) -> SignalFrame {
+        self.pulse.route(input, frequency)
+    }
+    fn ping(&mut self, probe: bool, hash: AttoRand) -> AttoRand {
+        self.pulse.ping(probe, hash).hash(Self::ID)
+    }
+}
+
+/// Pulse wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: pulse duty cycle in 0...1
+/// - Output 0: pulse wave
+#[inline]
+pub fn pulse<T: Float>() -> An<PulseWave<T>> {
+    An(PulseWave::new())
+}
+
+/*
+// Note. This causes compilation times to blow up with Rust 1.58.0.
+/// Pulse wave oscillator.
+/// - Input 0: frequency in Hz
+/// - Input 1: pulse duty cycle in 0...1
+/// - Output 0: pulse wave
+#[inline]
+pub fn pulse<T: Float>() -> An<
+    Pipe<
+        T,
+        Pipe<
+            T,
+            Stack<T, WaveSynth<'static, T, U2>, Pass<T, U1>>,
+            Stack<
+                T,
+                Pass<T, U1>,
+                Pipe<
+                    T,
+                    Binop<T, FrameAdd<T, U1>, Pass<T, U1>, Pass<T, U1>>,
+                    PhaseSynth<'static, T>,
+                >,
+            >,
+        >,
+        Binop<T, FrameSub<T, U1>, Pass<T, U1>, Pass<T, U1>>,
+    >,
+> {
+    (An(WaveSynth::<'static, T, U2>::new(DEFAULT_SR, &SAW_TABLE)) | pass())
+        >> (pass() | (pass() + pass()) >> An(PhaseSynth::<'static, T>::new(DEFAULT_SR, &SAW_TABLE)))
+        >> pass() - pass()
+}
+*/
